@@ -31,6 +31,7 @@
 #include "tgsi/tgsi_parse.h"
 #include "tgsi/tgsi_util.h"
 #include "tgsi/tgsi_dump.h"
+#include "tgsi/tgsi_strings.h"
 #include "lp_bld_debug.h"
 #include "lp_bld_tgsi.h"
 
@@ -113,17 +114,23 @@ analyse_tex(struct analysis_context *ctx,
       case TGSI_TEXTURE_1D:
          readmask = TGSI_WRITEMASK_X;
          break;
+      case TGSI_TEXTURE_1D_ARRAY:
       case TGSI_TEXTURE_2D:
       case TGSI_TEXTURE_RECT:
          readmask = TGSI_WRITEMASK_XY;
          break;
       case TGSI_TEXTURE_SHADOW1D:
+      case TGSI_TEXTURE_SHADOW1D_ARRAY:
       case TGSI_TEXTURE_SHADOW2D:
       case TGSI_TEXTURE_SHADOWRECT:
+      case TGSI_TEXTURE_2D_ARRAY:
       case TGSI_TEXTURE_3D:
       case TGSI_TEXTURE_CUBE:
          readmask = TGSI_WRITEMASK_XYZ;
          break;
+      case TGSI_TEXTURE_SHADOW2D_ARRAY:
+         readmask = TGSI_WRITEMASK_XYZW;
+         break;
       default:
          assert(0);
          return;
@@ -435,8 +442,12 @@ lp_build_tgsi_info(const struct tgsi_token *tokens,
             assert(size <= 4);
             if (ctx.num_imms < Elements(ctx.imm)) {
                for (chan = 0; chan < size; ++chan) {
-                  ctx.imm[ctx.num_imms][chan] =
-                        parse.FullToken.FullImmediate.u[chan].Float;
+                  float value = parse.FullToken.FullImmediate.u[chan].Float;
+                  ctx.imm[ctx.num_imms][chan] = value;
+
+                  if (value < 0.0f || value > 1.0f) {
+                     info->unclamped_immediates = TRUE;
+                  }
                }
                ++ctx.num_imms;
             }
